/***************************************************************************
*                                                                         *
*    This program is free software; you can redistribute it and/or modify *
*    it under the terms of the GNU General Public License as published by *
*    the Free Software Foundation; either version 2 of the License, or    *
*    (at your option) any later version.                                  *
*                                                                         *
* **************************************************************************/

#include <qregexp.h>
#include <qstring.h>

#include "word_fix.h"
#include "powerkadu.h"

#include "chat_manager.h"
#include "chat.h"
#include "debug.h"
#include "html_document.h"
#include "usergroup.h"
#include "config_dialog.h"
#include "misc.h"

#include <qlistview.h>
#include <qfile.h>

WordFix::WordFix(PowerKadu* pk)
{
	kdebugf();
	connect(chat_manager, SIGNAL(chatCreated(const UserGroup*)), this, SLOT(chatCreated(const UserGroup*)));
	connect(chat_manager, SIGNAL(chatDestroying(const UserGroup*)), this, SLOT(chatDestroying(const UserGroup*)));

	for ( uint i = 0; i < chat_manager->chats().count(); i++ )
		connectToChat(chat_manager->chats()[i]);

	// Loading list
	QString data = pk->cfg()->readEntry("PowerKadu", "WordFix_list", "");
	QStringList list = QStringList::split("\t\t", data);
	if (!list.count())
	{
		QFile defList(dataPath("kadu/modules/data/powerkadu/wf_default_list.data"));
		if (defList.open(IO_ReadOnly))
		{
			QTextStream s(&defList);
			QStringList pair;
			while (!s.atEnd())
			{
				pair = QStringList::split('|', s.readLine());
				if (pair.count() <= 0)
					continue;

				wordsList[pair[0]] = pair[1];
			}
			defList.close();
		}
		else
		{
			kdebug(QString("Can't open file: %1").arg(defList.name()));
		}
	}
	else
	{
		for ( uint i = 0; i < list.count(); i++ )
		{
			QStringList sp = QStringList::split('\t', list[i]);
			wordsList[sp[0]] = sp[1];
		}
	}

	// Config dialog
	ConfigDialog::addVGroupBox("PowerKadu", "PowerKadu", QT_TRANSLATE_NOOP("@default", "Words fix"));
		ConfigDialog::addVBox("PowerKadu", "Words fix", "Words fix VBOX");
			ConfigDialog::addListView("PowerKadu", "Words fix VBOX", QT_TRANSLATE_NOOP("@default", "Words to fix list:"));
			ConfigDialog::addHBox("PowerKadu", "Words fix VBOX", "Edit fields");
				ConfigDialog::addLineEdit2("PowerKadu", "Edit fields", QT_TRANSLATE_NOOP("@default", ""), QString::null,
					QT_TRANSLATE_NOOP("@default", "A word to be replaced."), "word");
				ConfigDialog::addLineEdit2("PowerKadu", "Edit fields", QT_TRANSLATE_NOOP("@default", ""), QString::null,
					QT_TRANSLATE_NOOP("@default", "Value to replace with."), "replace with");
			ConfigDialog::addHBox("PowerKadu", "Words fix VBOX", "Buttons");
				ConfigDialog::addPushButton("PowerKadu", "Buttons", QT_TRANSLATE_NOOP("@default", "Add"), QString::null,
					QT_TRANSLATE_NOOP("@default", "Adds above entered word and its value to the list."));
				ConfigDialog::addPushButton("PowerKadu", "Buttons", QT_TRANSLATE_NOOP("@default", "Change"), QString::null,
					QT_TRANSLATE_NOOP("@default", "Adds above entered word and its value to the list."));
				ConfigDialog::addPushButton("PowerKadu", "Buttons", QT_TRANSLATE_NOOP("@default", "Delete"), QString::null,
					QT_TRANSLATE_NOOP("@default", "Adds above entered word and its value to the list."));

	kdebugf2();
}

WordFix::~WordFix()
{
	kdebugf();
	disconnect(chat_manager, SIGNAL(chatCreated(const UserGroup*)), this, SLOT(chatCreated(const UserGroup*)));
	disconnect(chat_manager, SIGNAL(chatDestroying(const UserGroup*)), this, SLOT(chatDestroying(const UserGroup*)));

	for ( uint i = 0; i < chat_manager->chats().count(); i++ )
		disconnectFromChat(chat_manager->chats()[i]);

	QStringList list;
	for ( uint i = 0; i < wordsList.keys().count(); i++ )
	{
		QString word = wordsList.keys()[i];
		list.append(word+"\t"+wordsList[word]);
	}
	powerKadu->cfg()->writeEntry("PowerKadu", "WordFix_list", list.join("\t\t"));

	// Config dialog
	ConfigDialog::removeControl("PowerKadu", "Delete");
	ConfigDialog::removeControl("PowerKadu", "Change");
	ConfigDialog::removeControl("PowerKadu", "Add");
	ConfigDialog::removeControl("PowerKadu", "Buttons");
	ConfigDialog::removeControl("PowerKadu", "", "replace with");
	ConfigDialog::removeControl("PowerKadu", "", "word");
	ConfigDialog::removeControl("PowerKadu", "Words to fix list:");
	ConfigDialog::removeControl("PowerKadu", "Words fix");

	kdebugf2();
}

void WordFix::chatCreated(const UserGroup* group)
{
	kdebugf();
	connectToChat(chat_manager->findChat(group));
	kdebugf2();
}

void WordFix::chatDestroying(const UserGroup* group)
{
	kdebugf();
	disconnectFromChat(chat_manager->findChat(group));
	kdebugf2();
}

void WordFix::connectToChat(Chat* chat)
{
	kdebugf();
	connect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(sendRequest(Chat*)));
	kdebugf2();
}

void WordFix::disconnectFromChat(Chat* chat)
{
	kdebugf();
	disconnect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(sendRequest(Chat*)));
	kdebugf2();
}

void WordFix::sendRequest(Chat* chat)
{
	kdebugf();

	// Reading chat input to html document.
	HtmlDocument doc;
	doc.parseHtml( chat->edit()->text() );

	// Parsing and replacing.
	for ( int i = 0; i < doc.countElements(); i++ )
	{
		if (!doc.isTagElement(i))
			doReplace( doc.elementText(i) );
	}

	// Putting back corrected text.
	chat->edit()->setText( doc.generateHtml() );

	kdebugf2();
}

void WordFix::doReplace(QString &text)
{
	kdebugf();

	// Make text as " "+text+" ", to match words at the start and end of text.
	QString txt = text;
	txt = " "+txt+" ";

	// Replacing
	QString key;
	for ( uint i = 0; i < wordsList.keys().count(); i++ )
	{
		key = wordsList.keys()[i];
		txt.replace(
				QRegExp(
						"[\\s\\!\\.\\,\\(\\)\\[\\]\\;\\:\\\"\\']{1}"
						+key
						+"[\\s\\!\\.\\,\\(\\)\\[\\]\\;\\:\\\"\\']{1}"
					),
				" "+wordsList[key]+" "
			);
	}

	// Cutting off " " from start and end.
	txt = txt.mid(1, txt.length()-2);
	text = txt;
	kdebugf2();
}

void WordFix::onCreateConfig()
{
	kdebugf();

	QListView* list = ConfigDialog::getListView("PowerKadu", "Words to fix list:");
	QPushButton* add = ConfigDialog::getPushButton("PowerKadu", "Add");
	QPushButton* change = ConfigDialog::getPushButton("PowerKadu", "Change");
	QPushButton* del = ConfigDialog::getPushButton("PowerKadu", "Delete");
	QLineEdit* word = ConfigDialog::getLineEdit("PowerKadu", "", "word");
	QLineEdit* value = ConfigDialog::getLineEdit("PowerKadu", "", "replace with");
	QVBox* vbox = ConfigDialog::getVBox("PowerKadu", "Words fix VBOX");

	vbox->setMaximumWidth( 500 );
	list->setAllColumnsShowFocus( true );

	list->addColumn( QT_TRANSLATE_NOOP("@default", "Word"), 250 );
	list->addColumn( QT_TRANSLATE_NOOP("@default", "Replace with"), 246 - list->verticalScrollBar()->width() );


	QListViewItem* item;
	for ( uint i = 0; i < wordsList.keys().count(); i++ )
	{
		QString wordStr = wordsList.keys()[i];
		item = new QListViewItem( list );
		item->setText( 0, wordStr );
		item->setText( 1, wordsList[wordStr] );
	}

	connect(list, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(wordSelected(QListViewItem*)));
	connect(change, SIGNAL(clicked()), this, SLOT(changeSelected()));
	connect(del, SIGNAL(clicked()), this, SLOT(deleteSelected()));
	connect(add, SIGNAL(clicked()), this, SLOT(addNew()));
	connect(word, SIGNAL(returnPressed()), this, SLOT(moveToNewValue()));
	connect(value, SIGNAL(returnPressed()), this, SLOT(addNew()));

	change->setEnabled( false );
	del->setEnabled( false );

	kdebugf2();
}

void WordFix::onDestroyConfig()
{
	kdebugf();

	QListView* list = ConfigDialog::getListView("PowerKadu", "Words to fix list:");
	QPushButton* add = ConfigDialog::getPushButton("PowerKadu", "Add");
	QPushButton* change = ConfigDialog::getPushButton("PowerKadu", "Change");
	QPushButton* del = ConfigDialog::getPushButton("PowerKadu", "Delete");
	QLineEdit* word = ConfigDialog::getLineEdit("PowerKadu", "", "word");
	QLineEdit* value = ConfigDialog::getLineEdit("PowerKadu", "", "replace with");

	disconnect(list, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(wordSelected(QListViewItem*)));
	disconnect(change, SIGNAL(clicked()), this, SLOT(changeSelected()));
	disconnect(del, SIGNAL(clicked()), this, SLOT(deleteSelected()));
	disconnect(add, SIGNAL(clicked()), this, SLOT(addNew()));
	disconnect(word, SIGNAL(returnPressed()), this, SLOT(moveToNewValue()));
	disconnect(value, SIGNAL(returnPressed()), this, SLOT(addNew()));

	kdebugf2();
}

void WordFix::applyConfig()
{
	kdebugf();

	wordsList.clear();
	QListView* list = ConfigDialog::getListView("PowerKadu", "Words to fix list:");
	QListViewItem* item = list->firstChild();
	QString wordStr = item->text( 0 ),
			valueStr = item->text( 1 );

	wordsList[wordStr] = valueStr;

	while ( item = item->itemBelow() )
	{
		wordStr = item->text( 0 );
		valueStr = item->text( 1 );
		wordsList[wordStr] = valueStr;
	}

	kdebugf2();
}

void WordFix::wordSelected(QListViewItem* item)
{
	kdebugf();

	QPushButton* change = ConfigDialog::getPushButton("PowerKadu", "Change");
	QPushButton* del = ConfigDialog::getPushButton("PowerKadu", "Delete");
	QLineEdit* word = ConfigDialog::getLineEdit("PowerKadu", "", "word");
	QLineEdit* value = ConfigDialog::getLineEdit("PowerKadu", "", "replace with");

	if (!item)
	{
		change->setEnabled( false );
		del->setEnabled( false );
		return;
	}
	else
	{
		change->setEnabled( true );
		del->setEnabled( true );
	}

	word->setText(item->text( 0 ));
	value->setText(item->text( 1 ));

	kdebugf2();
}

void WordFix::changeSelected()
{
	kdebugf();
	QLineEdit* word = ConfigDialog::getLineEdit("PowerKadu", "", "word");
	QLineEdit* value = ConfigDialog::getLineEdit("PowerKadu", "", "replace with");
	QListView* list = ConfigDialog::getListView("PowerKadu", "Words to fix list:");

	QListViewItem* item = list->currentItem();
	if (!item)
		return;

	QString wordStr = word->text(),
			valueStr = value->text();

	item->setText( 0, wordStr );
	item->setText( 1, valueStr );
	kdebugf2();
}

void WordFix::deleteSelected()
{
	kdebugf();
	QLineEdit* word = ConfigDialog::getLineEdit("PowerKadu", "", "word");
	QLineEdit* value = ConfigDialog::getLineEdit("PowerKadu", "", "replace with");
	QListView* list = ConfigDialog::getListView("PowerKadu", "Words to fix list:");
	QPushButton* change = ConfigDialog::getPushButton("PowerKadu", "Change");
	QPushButton* del = ConfigDialog::getPushButton("PowerKadu", "Delete");

	QListViewItem* item = list->currentItem();
	if (!item)
		return;

	QString wordStr = item->text( 0 );

	delete item;
	word->setText("");
	value->setText("");

	change->setEnabled( false );
	del->setEnabled( false );
	list->setCurrentItem( 0 );

	kdebugf2();
}

void WordFix::addNew()
{
	kdebugf();
	QLineEdit* word = ConfigDialog::getLineEdit("PowerKadu", "", "word");
	QLineEdit* value = ConfigDialog::getLineEdit("PowerKadu", "", "replace with");
	QListView* list = ConfigDialog::getListView("PowerKadu", "Words to fix list:");
	QPushButton* change = ConfigDialog::getPushButton("PowerKadu", "Change");
	QPushButton* del = ConfigDialog::getPushButton("PowerKadu", "Delete");

	QString wordStr = word->text(),
			valueStr = value->text();

	if (wordStr.isEmpty())
		return;

	QListViewItem* item = new QListViewItem( list );
	item->setText( 0, wordStr );
	item->setText( 1, valueStr );
	list->sort();
	list->ensureItemVisible( item );

	word->setText("");
	value->setText("");

	change->setEnabled( false );
	del->setEnabled( false );

	kdebugf2();
}

void WordFix::moveToNewValue()
{
	kdebugf();
	QLineEdit* value = ConfigDialog::getLineEdit("PowerKadu", "", "replace with");
	value->setFocus();
	kdebugf2();
}
